<?php
/**
 * CodeIgniter Portlet
 *
 * PHP version 5
 *
 * @package portlet
 * @version $Id: modify_office_files.php 5057 2012-11-26 15:45:45Z christian $
*/

require_once APPPATH.'us_libraries/Portlet'.EXT;
require_once APPPATH.'us_libraries/Widget_interface'.EXT;
require_once APPPATH.'us_libraries/Widget_editable_interface'.EXT;

/**
 * A portlet to connect Modify Office Files
 */
class Com_untermstrich_modify_office_files extends Portlet implements Widget_interface, Widget_editable_interface
{
    const DEFAULT_LOG4J = false;
    const DEFAULT_PORT = '17332';
    const DEFAULT_LOCAL = true;
    const DEFAULT_ACTIVE = true;
    const ERROR_CANNOT_CONNECT = ' (Cannot connect to Modify office files Server)';
    const MIN_MAJOR = 1;
    const MIN_MINOR = 2;
    
    /**
     * Get portlet version
     * 
     * @return  integer
     */
    public static function get_portlet_version()
    {
        return 23;
    }
    
    /**
     * Shows the next birthdays of the staff members
     *
     * DO NOT USE XAJAX WITHIN THIS METHOD
     *
     * @param   array   $params     Optional The parameters
     * @return  string  The page
     *
     * @uses US_Smarty
     */
    public static function view(array $params=array())
    {
        //Libraries
        self::$CI->load->registry_load_model('System_settings_model', 'system_settings', true);
        self::$CI->smarty->assign_lang_info();

        if (!self::server_active()) {
            return '<span style="font-size:200%;">&#9757;</span> '.self::$CI->lang->line('com_untermstrich_modify_office_files_server_config').self::write();
        }
        
        $json_path = self::_get_json_path();
        if (!$json_path) {
            return '<span style="font-size:200%;">&#9757;</span> '.self::$CI->lang->line('com_untermstrich_modify_office_files_save_config').self::write();
        }
        
        self::$CI->smarty->assign('server_path1', 'cd '.self::$portlet_dir);
        self::$CI->smarty->assign('server_path2', self::_create_send_to_jar_command('server', $json_path, ""));
        self::$CI->smarty->assign('server_version', self::is_server_running());
        self::$CI->smarty->assign('is_windows', (strtoupper(substr(PHP_OS, 0, 3))==='WIN'));
        return self::$CI->smarty->view('start_server', true, self::$portlet_dir.'views/').self::write();
    }
    
    /**
     * Processes xajax calls used by the portlet
     *
     * @param   Page    $the_page   The page the portlet is added to
     * @return  void
     */
    public static function process(Page $the_page)
    {
    }
    
    /**
     * Process file using Modify office file
     * 
     * @param   string  $action             Action to call
     * @param   string  $fullpath_from      File from
     * @param   string  $fullpath_to        File to
     * @param   string  $password           Password
     * @param   string  $fullpath_xml       Path to xml
     * @return  string
     */
    public static function process_file($action, $fullpath_from, $fullpath_to, $password="", $fullpath_xml=false)
    {
        if (self::server_active()) {
            $ret = self::_send_to_server($action, $fullpath_from, $fullpath_to, $password, $fullpath_xml);
        } else {
            $ret = self::_send_to_jar($action, $fullpath_from, $fullpath_to, $password, $fullpath_xml);
        }
        
        switch ($action) {
            case 'xlsxdoc':
            case 'xlsxfillcsv':
            case 'xlsxfill':
                if (isset(self::$CI->xajax) && strlen($ret)>4 && substr($ret, -4)==='DONE') {
                    self::$CI->xajax->message('WARNING: (modify_office_files): '."\n".$ret, US_Xajax_response::MSGM_WARNING);
                    return 'DONE';
                }
                break;
        }
        
        return $ret;
    }
    
    /**
     * Is the server running and up to date?
     * 
     * @return  array
     */
    public static function is_server_running()
    {
        $msg = self::_send_to_server('version', self::_get_json_path(), '');
        
        //Get version parts
        $version_parts = explode('.', $msg);
        if (count($version_parts)!=3) {
            return array($msg, false);
        }
        
        //Check major version
        if (intval($version_parts[0])<self::MIN_MAJOR) {
            return array(self::$CI->lang->line('com_untermstrich_modify_office_files_old_version').$msg, false);
        } elseif (intval($version_parts[0])<self::MIN_MAJOR) {
            return array($msg, true);
        }

        //Check minor version
        if (intval($version_parts[1])<self::MIN_MINOR) {
            return array(self::$CI->lang->line('com_untermstrich_modify_office_files_old_version').$msg, false);
        }

        return array($msg, true);
    }
    
    /**
     * Process file using Modify office file JAR
     * 
     * @param   string  $action             Action to call
     * @param   string  $fullpath_from      File from
     * @param   string  $fullpath_to        File to
     * @param   string  $password           Password
     * @param   string  $fullpath_xml       Path to xml
     * @return  string
     */
    private static function _send_to_jar($action, $fullpath_from, $fullpath_to, $password="", $fullpath_xml=false)
    {
        //For Linux only
        if (strtoupper(substr(PHP_OS, 0, 3)) !== 'WIN' && strtoupper(substr(PHP_OS, 0, 6)) !== 'DARWIN') {
            $old_locale = setlocale(LC_ALL, 0);
            $old_env = getenv('LC_ALL');
            @setlocale(LC_ALL, 'en_US.UTF-8');
            @putenv('LC_ALL=en_US.UTF-8');
            #exec('locale charmap');
        }
        
        $cmd = 'cd '.self::$portlet_dir.' && '.self::_create_send_to_jar_command($action, $fullpath_from, $fullpath_to, $password, $fullpath_xml);
        log_message('debug', 'MODIFY OFFICE: '.$cmd);
        
        //Call
        $ret = @shell_exec($cmd.' 2>&1');
        
        //For Linux only
        if (strtoupper(substr(PHP_OS, 0, 3))!=='WIN' && strtoupper(substr(PHP_OS, 0, 6))!=='DARWIN') {
            @setlocale(LC_ALL, $old_locale);
            @putenv('LC_ALL='.$old_env);
            #exec('locale charmap');             
        }
        
        return trim($ret);
    }
    
    /**
     * Create Modify office file JAR call
     * 
     * @param   string  $action             Action to call
     * @param   string  $fullpath_from      File from
     * @param   string  $fullpath_to        File to
     * @param   string  $password           Password
     * @param   string  $fullpath_xml       Path to xml
     * @return  string
     */
    private static function _create_send_to_jar_command($action, $fullpath_from, $fullpath_to, $password="", $fullpath_xml=false)
    {
        $widget_data = self::get_widget_config();
        
        //Get path to modify_office_files
        $bin = realpath(self::$portlet_dir.'portlets/modify_office_files.jar');
        
        //Basic variables
        $java_path = self::$CI->config->item('us_java_path');
        $java_part = $java_path;
        if ($widget_data['log4j']) {
            $java_part .= ' -Dlogger=Log4j';
        }
        $java_part .= ' -Xmx512m -Djava.awt.headless=true -jar '.escapeshellarg($bin);
        
        //Create command
        $cmd = $java_part.' '.escapeshellarg($action).' '.escapeshellarg($fullpath_from).' '.escapeshellarg($fullpath_to).' '.escapeshellarg($password);
        if ($fullpath_xml!==false) {
            $cmd .= ' '.escapeshellarg($fullpath_xml);
        }
        return $cmd;
    }
    
    /**
     * Process file using Modify office file server
     * 
     * @param   string  $action             Action to call
     * @param   string  $fullpath_from      File from
     * @param   string  $fullpath_to        File to
     * @param   string  $password           Password
     * @param   string  $fullpath_xml       Path to xml
     * @return  string
     */
    private static function _send_to_server($action, $fullpath_from, $fullpath_to, $password="", $fullpath_xml=false)
    {
        $widget_data = self::get_widget_config();
        
        if (us_use_latin1_access()) {
            //If the use latin1 mode is active, we might have to encode the file names to UTF-8
            if (function_exists('mb_detect_encoding')) {
                //If we have mbstring available, we use mb_detect_encoding to discover, if we have a ISO-8859-1 string.
                if (mb_detect_encoding($fullpath_from.$fullpath_to, 'ISO-8859-1', true)) {
                    $fullpath_from = utf8_encode($fullpath_from);
                    $fullpath_to = utf8_encode($fullpath_to);
                }
            } else {
                //Without mbstring, alway UTF-8 encode the file names
                $fullpath_from = utf8_encode($fullpath_from);
                $fullpath_to = utf8_encode($fullpath_to);
            }
        }
        
        //Libraries
        $url = '/'.$action.'?file_from='.urlencode($fullpath_from).'&file_to='.urlencode($fullpath_to).'&password='.urlencode($password);
        if ($fullpath_xml!==false) {
            $url .= '&xml_file='.urlencode($fullpath_xml);
        }

        //Create HMAC
        $time = time();
        $url .= '&time='.$time;
        $forHmac = $action.'|'.urlencode($fullpath_from).'|'.urlencode($fullpath_to).'|'.urlencode($password).'|'.urlencode($fullpath_xml?$fullpath_xml:'').'|'.urlencode($time);
        $url .= '&hmac='.hash_hmac('sha256', urldecode($forHmac), $widget_data['token']);

        $ok = self::_send_to_server_get('localhost', $widget_data['port'], $url);
        if ($ok===false) {
            return self::$CI->lang->line('com_untermstrich_modify_office_files_no_conn').self::ERROR_CANNOT_CONNECT;
        } else {
            return trim($ok);
        }
    }
    
    /**
     * Send to server via get
     * 
     * @param   string  $host   Host
     * @param   int     $port   Port
     * @param   string  $url    URL part from /
     * @return  mixed
     */
    private static function _send_to_server_get($host, $port, $url)
    {
        $file = @fsockopen($host, $port, $errno, $errstr, 8);
        log_message('error', 'fsockopen '.$host.':'.$port);
        if (!$file) {
            log_message('error', __METHOD__.' failed: Cannot fsockopen '.$host.':'.$port);
            return false;
        }
        
        //Send header
        log_message('error', 'fsockopen send GET:'.$url);
        if (!@fputs($file, "GET $url HTTP/1.0\r\n")) {
            log_message('error', __METHOD__.' failed: Cannot fsockopen post header '.$host.':'.$port);
            return false;
        }
        @fputs($file, "Connection: close\r\n\r\n");
        
        //Read lines - download
        $reply = '';
        while ($line = @fgets($file, 1024)) {
            //On error
            if (false===$line) {
                log_message('error', __METHOD__.' failed: Cannot read (@fgets) '.$url);
                return false;
            }

            $reply .= $line;
        }

        //Close source file
        @fclose($file);
        
        //Remove header
        $pos = strpos($reply, "\r\n\r\n");
        $reply = substr($reply, $pos + 4);
        
        return $reply;
    }

    /**
     * Get a config page for the widget XAJAX
     *
     * @param   array   $params   Diffent params
     * @return  string
     */    
    public static function get_widget_configpage(array $params = array())
    {
        self::$CI->smarty->assign_lang_info();
        
        self::$CI->smarty->assign('widget_data', self::get_widget_config());
        return self::$CI->smarty->view('server_config', true, self::$portlet_dir.'views/').self::write();
    }

    /**
     * Get information about the widget
     *
     * @return  object
     */
    public static function get_widget_information()
    {
        $reply = new stdClass();
        $reply->main_title = 'basic_correspondence';
        $reply->title = 'com_untermstrich_modify_office_files_title';
        $reply->link = null;
        $reply->help_link = ':basics:basics:butler:modify_office_files';
        $reply->removable = true;
        $reply->right = new stdClass();
        $reply->right->module = Login_model::R_MASTERDATAMANAGEMENT;
        $reply->right->level = Login_model::L_MANAGEMENTPLUS;
        $reply->print = false;
        return $reply;
    }

    /**
     * Save a config page for the widget XAJAX
     *
     * @param   array   $params   Different params
     * @return  string
     */
    public static function save_configpage(array $params = array())
    {
        //Libraries
        self::$CI->load->registry_load_model('System_settings_model', 'system_settings', true);
        
        $active = self::$CI->input->post('modify_office_files_active')==='TRUE'? true : false;
        $log4j = self::$CI->input->post('modify_office_files_log4j')==='TRUE'? true : false;
        $port = intval(self::$CI->input->post('modify_office_files_port'));
        
        //Prepare object to save
        $widget_data = self::get_widget_config();
        $widget_data['log4j'] = $log4j;
        $widget_data['active'] = $active;
        $widget_data['port'] = $port;
        $widget_data['local'] = true;
        
        //Save to systemsettings
        self::$CI->system_settings->setModifyOfficeFilesConfig(@serialize($widget_data));
        
        //Save config to conf.json file
        self::write_config_file();
    }
    
    /**
     * Get a config for the widget
     *
     * @return  array
     */
    private static function get_widget_config()
    {
        //Libraries
        self::$CI->load->registry_load_model('System_settings_model', 'system_settings', true);

        $widget_data = @unserialize(self::$CI->system_settings->getModifyOfficeFilesConfig());

        if (!isset($widget_data) || !is_array($widget_data)) {
            $widget_data = array();
        }
        
        if (!isset($widget_data['log4j'])) {
            $widget_data['log4j'] = self::DEFAULT_LOG4J;
        }
        if (!isset($widget_data['active'])) {
            $widget_data['active'] = self::DEFAULT_ACTIVE;
        }
        if (!isset($widget_data['port']) || empty($widget_data['port'])) {
            $widget_data['port'] = self::DEFAULT_PORT;
        }      
        if (!isset($widget_data['local'])) {
            $widget_data['local'] = self::DEFAULT_LOCAL;
        }
        if (!isset($widget_data['token']) || empty($widget_data['token'])) {
            $widget_data['token'] = self::get_rand_token();
        }
        
        return $widget_data;
    }
    
    /**
     * Should the server be active
     * 
     * @return  boolean
     */
    private static function server_active()
    {
        //Libraries
        self::$CI->load->registry_load_model('System_settings_model', 'system_settings', true);

        $widget_data = @unserialize(self::$CI->system_settings->getModifyOfficeFilesConfig());

        if (!isset($widget_data) || !is_array($widget_data)) {
            return false;
        }
        
        if (!isset($widget_data['active']) || empty($widget_data['active'])) {
            return false;
        }
        
        return $widget_data['active'];
    }
    
    /**
     * Get rand token
     * 
     * @return  string
     */
    private static function get_rand_token()
    {
        if (method_exists(self::$CI->security,'get_token')) {
            return self::$CI->security->get_token();
        } else {
            return sha1(uniqid(mt_rand(0, mt_getrandmax()), true));
        }
    }
    
    /**
     * Write the node server config file conf.json
     * 
     * @return void
     */    
    public static function write_config_file()
    {
        self::$CI->load->registry_load_model('System_settings_model', 'system_settings', true);
        
        $widget_data = self::get_widget_config(); 

        self::$CI->load->library('Project_directories_tools', null, 'project_directories_tools');
        self::$CI->project_directories_tools->ensure_path_exists(self::get_path('WRITEABLE'), true);
        
        //Save config to conf.json file
        file_put_contents(self::get_path('WRITEABLE').'conf.json', us_json_encode($widget_data));  
    }
    
    /**
     * Get path
     * 
     * @param   string    $type       The path to get
     * @return  string
     */
    private static function get_path($type) 
    {
        $paths = array(
            'PORTLET' => realpath(dirname(__FILE__)).'/'
        );
        $paths['ROOT'] = $paths['PORTLET'].'../';
        $paths['WRITEABLE'] = $paths['ROOT'].'writeable/';
        
        return $paths[strtoupper($type)];
    }
    
    /**
     * Get json file path
     * 
     * @return  string
     */
    private static function _get_json_path()
    {
        return realpath(self::get_path('WRITEABLE').'conf.json');
    }

} // END class Com_untermstrich_modify_office_files